with Interfaces, Interfaces.C, Interfaces.C.Strings;
with Ada;
with Ada.Unchecked_Conversion;
with Ada.Exceptions;
with System;

with Smuw_Snapshot;
with Smu_Proxy, Smu_vTable;

with Kcg_Config;
with Kcg_Top;

package body Smuw_Interface
is
    procedure Restore_Interface is
    begin
        U := U_restore;
        Ctx := Ctx_restore;
        null;
    end Restore_Interface;

    procedure Execute_Interface is
    begin
        Smu_Proxy.pSimulator.fn_acquire_value_mutex(Smu_Proxy.pSimulator);
        U_execute := U;
        Smu_Proxy.pSimulator.fn_release_value_mutex(Smu_Proxy.pSimulator);
        null;
    end Execute_Interface;

    function Ssm_Init return Interfaces.C.Int is
    begin
        Restore_Interface;
        -- Call to init function
        Kcg_Top.M3C3_Init (
            Ctx => Ctx
        );
        return 1;
    end Ssm_Init;

    function Ssm_Reset return Interfaces.C.Int is
    begin
        Restore_Interface;
        -- Call to reset function
        Kcg_Top.M3C3_Reset (
            Ctx => Ctx
        );
        return 1;
    end Ssm_Reset;

    function Ssm_Custom_Init return Interfaces.C.Int is
        nRet: Interfaces.C.Int := 0;
        use Smu_vTable;
    begin
        if vtable /= NULL and then vtable.m_pfnCustomInit /= NULL then
            -- VTable function provided => call it 
            nRet := vtable.m_pfnCustomInit (pOutCtx => Ctx'Address);
        else
            -- VTable misssing => error
            nRet := 0;
        end if;
        return nRet;
    end Ssm_Custom_Init;

    function Ssm_Step return Interfaces.C.Int is
    begin
        strExceptionMsg := Ada.Strings.Unbounded.Null_Unbounded_String;
        Execute_Interface;
        -- Call to cycle function
        Kcg_Top.M3C3 (
            U => U_execute,
            Ctx => Ctx);
        return 1;
        exception
            when An_Exception : others => 
              Ada.Strings.Unbounded.Append(strExceptionMsg, Ada.Exceptions.Exception_Name(An_Exception));
              Ada.Strings.Unbounded.Append(strExceptionMsg, ": ");
              Ada.Strings.Unbounded.Append(strExceptionMsg, Ada.Exceptions.Exception_Message(An_Exception));
              return 0;
    end Ssm_Step;

    function Ssm_Stop return Interfaces.C.Int is
    begin
        return 1;
    end Ssm_Stop;

    function Ssm_Get_Default_Real_Format return Interfaces.C.Strings.chars_ptr is
    begin
        -- from .etp:
        return Interfaces.C.Strings.New_String("%.5g");
    end Ssm_Get_Default_Real_Format;

    type Dump_Ctx is record
        U: Kcg_Config.Kcg_Float64;
        Ctx: Kcg_Top.Context_M3C3;
    end record;

    Dump_Size: Integer :=  Dump_Ctx'Size / Interfaces.C.unsigned_char'Size;
    type Byte_Array is array (Integer range <>) of Interfaces.C.unsigned_char;
    subtype Dump_Data is Byte_Array (1..Dump_Size);

    function Ssm_Get_Dump_Size return Interfaces.C.Int is
    begin
        return Interfaces.C.int(Dump_Size);
    end Ssm_Get_Dump_Size;

    function To_DumpData is new Ada.Unchecked_Conversion (Source => Dump_Ctx, Target => Dump_Data);
    function From_DumpData is new Ada.Unchecked_Conversion (Source => Dump_Data, Target => Dump_Ctx);

    procedure Ssm_Gather_Dump_Data(pData: System.Address) is
        smu_dump_data: Dump_Data; for smu_dump_data'Address use pData;
        smu_dump_ctx: Dump_Ctx;
    begin
        smu_dump_ctx.U := U;
        smu_dump_ctx.Ctx := Ctx;
        smu_dump_data := To_DumpData(smu_dump_ctx);
    end Ssm_Gather_Dump_Data;

    procedure Ssm_Restore_Dump_Data(pData: System.Address) is
        smu_dump_data:Dump_Data; for smu_dump_data'Address use pData;
        smu_dump_ctx: Dump_Ctx;
    begin
        smu_dump_ctx := From_DumpData(smu_dump_data);
        U := smu_dump_ctx.U;
        Ctx := smu_dump_ctx.Ctx;
    end Ssm_Restore_Dump_Data;

    function Ssm_Save_Snapshot(szFilePath: Interfaces.C.Strings.chars_ptr; nCycle: Interfaces.C.size_t) return Interfaces.C.int is
        bOK: Boolean := False;
        iCycle: Integer := Integer(nCycle);
    begin
        Smuw_Snapshot.save_snapshot(Interfaces.C.Strings.Value(szFilePath), iCycle,
            U,
            Ctx,
            bOK);
        if bOK then
            return 1;
        else
            return 0;
        end if;
    exception
        when others => return 0;
    end Ssm_Save_Snapshot;

    function Ssm_Load_Snapshot(szFilePath: Interfaces.C.Strings.chars_ptr; nCycle: access Interfaces.C.size_t) return Interfaces.C.int is
        bOK: Boolean := False;
        iCycle: Integer;
    begin
        Smuw_Snapshot.load_snapshot(Interfaces.C.Strings.Value(szFilePath), iCycle,
            U,
            Ctx,
            bOK);
        if bOK then
            nCycle.all := Interfaces.C.size_t(iCycle);
            return 1;
        else
            return 0;
        end if;
    exception
        when others => return 0;
    end Ssm_Load_Snapshot;

    function Ssm_Get_CheckSum return Interfaces.C.Strings.chars_ptr is
    begin
        return Interfaces.C.Strings.New_String("427c870dc4fbd561916b6585029459c3");
    end Ssm_Get_CheckSum;

    function Ssm_Get_Smu_Types_CheckSum return Interfaces.C.Strings.chars_ptr is
    begin
        return Interfaces.C.Strings.New_String("3093108411c6570b9c21d7b04ee1db2c");
    end Ssm_Get_Smu_Types_CheckSum;

    function Ssm_Get_Exception_Msg return Interfaces.C.Strings.chars_ptr is
    begin
        return Interfaces.C.Strings.new_string(Ada.Strings.Unbounded.To_String(strExceptionMsg));
    end Ssm_Get_Exception_Msg;

end Smuw_Interface;
