with Kcg_Config;
with Kcg_Top;
with linear;
with System;
with Ada.Text_IO;
with Ada.Text_IO.Unbounded_IO;
with Ada.Strings.Unbounded;
with Ada.Characters.Latin_1; use Ada.Characters.Latin_1;
with Interfaces, Interfaces.C, Interfaces.C.Strings;
with System.Storage_Elements; use System.Storage_Elements;
with Ada.Strings.Fixed;
with Smu_Types;
with Smu_Lib;
with Smuw_Type;

package body Smuw_Snapshot
is

    procedure save_snapshot(filePath: in String; nCycle: in Integer;
        U: in Kcg_Config.Kcg_Float64;
        Ctx: in Kcg_Top.Context_M3C3;
        bOK: out Boolean) is
        snsFile: Ada.Text_IO.File_Type;
        savRealFormat: Interfaces.C.char_array (1..10);
        count: Interfaces.C.size_t;
    begin
        bOK := True;
        Ada.Text_IO.Create(snsFile, Ada.Text_IO.Out_File, filePath);
        savRealFormat := Smu_Types.converter.m_RealFormat;
        Interfaces.C.To_C("%.17g", Smu_Types.converter.m_RealFormat, count);
        save_var("U", U'Address, U'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
        Smuw_Kcg_Top.Context_M3C3.save("Ctx", Ctx'Address, Ctx'Address, snsFile, bOK);

        Ada.Text_IO.Close(snsFile); 
        Smu_Types.converter.m_RealFormat := savRealFormat;
    end save_snapshot;

    procedure load_snapshot(filePath: in String; nCycle: out Integer;
        U: out Kcg_Config.Kcg_Float64;
        Ctx: out Kcg_Top.Context_M3C3;
        bOK: out Boolean) is
       snsFile: Ada.Text_IO.File_Type;
    begin
        bOK:= True;
        nCycle:= 1;--TODO
        Ada.Text_IO.Open(snsFile, Ada.Text_IO.In_File, filePath);
        load_var(U'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
        Smuw_Kcg_Top.Context_M3C3.load(Ctx, snsFile, bOK);

        Ada.Text_IO.Close(snsFile); 
    end load_snapshot;

    procedure save_var(adaPath: in String; pValue: in System.Address; pBaseAddr: in System.Address; pfnTypeToString: Smu_Types.PFN_TYPE_TO_STRING; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
        strValue: aliased Smu_Lib.Sim_String;
        use type Interfaces.C.int;
        nOffset: Integer_Address:= System.Storage_Elements.To_Integer(pValue)-System.Storage_Elements.To_Integer(pBaseAddr);
    begin
        Smu_Lib.Sim_String_Alloc(strValue'Access);
        bOK := bOK and (pfnTypeToString(pValue, Smu_Lib.Sim_String_Append'Access, strValue'Address) /= 0);
        Ada.Text_IO.Put(snsFile, Interfaces.C.Strings.Value(Smu_Lib.Sim_String_Get(strValue'Access)) & HT & "[offset="&Ada.Strings.Fixed.Trim(nOffset'Img, Ada.Strings.Left)&"]" & HT & adaPath & LF);
        Smu_Lib.Sim_String_Free(strValue'Access);
    end save_var;

    procedure save_field(adaPath: in String; proj: in String; pValue: in System.Address; pBaseAddr: in System.Address; pfnTypeToString: Smu_Types.PFN_TYPE_TO_STRING; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
        strAdaPath: aliased Smu_Lib.Sim_String;
        use type Interfaces.C.int;
    begin
        Smu_Lib.Sim_String_Alloc(strAdaPath'Access);
        bOK := bOK and (Smu_Lib.Sim_String_Append(Interfaces.C.Strings.new_string(adaPath), strAdaPath'Address)/= 0);
        bOK := bOK and (Smu_Lib.Sim_String_Append(Interfaces.C.Strings.new_string(proj), strAdaPath'Address)/= 0);
        save_var(Interfaces.C.Strings.Value(Smu_Lib.Sim_String_Get(strAdaPath'Access)), pValue, pBaseAddr, pfnTypeToString, snsFile, bOK);
        Smu_Lib.Sim_String_Free(strAdaPath'Access);
    end save_field;

    procedure save_context(adaPath: in String; proj: in String; pValue: in System.Address; pBaseAddr: in System.Address; pfnSaveContext: PFN_SAVE_CONTEXT; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
        strAdaPath: aliased Smu_Lib.Sim_String;
        use type Interfaces.C.int;
    begin
        Smu_Lib.Sim_String_Alloc(strAdaPath'Access);
        bOK := bOK and (Smu_Lib.Sim_String_Append(Interfaces.C.Strings.new_string(adaPath), strAdaPath'Address)/= 0);
        bOK := bOK and (Smu_Lib.Sim_String_Append(Interfaces.C.Strings.new_string(proj), strAdaPath'Address)/= 0);
        pfnSaveContext(Interfaces.C.Strings.Value(Smu_Lib.Sim_String_Get(strAdaPath'Access)), pValue, pBaseAddr, snsFile, bOK);
        Smu_Lib.Sim_String_Free(strAdaPath'Access);
    end save_context;

    procedure read_line(snsFile: in Ada.Text_IO.File_Type; strLine : out Ada.Strings.Unbounded.Unbounded_String; bOK: in out Boolean) is
        Buffer_Size : constant := 2000;
        strItem: String(1 ..Buffer_Size);
        Last: Natural;
        use type Ada.Strings.Unbounded.Unbounded_String;
    begin
        strLine := Ada.Strings.Unbounded.Null_Unbounded_String;
        loop
            Ada.Text_IO.Get_Line(snsFile, strItem, Last);
            if Last = 0 then
                null;
            elsif(Last>=2 and strItem(Last-1) = CR and strItem(Last) = LF) then
                Ada.Strings.Unbounded.Append(strLine, strItem(1 .. (Last - 2)));
            elsif(Last>=1 and strItem(Last) = LF) then
                Ada.Strings.Unbounded.Append(strLine, strItem(1 .. (Last - 1)));
            else
                Ada.Strings.Unbounded.Append(strLine, strItem(1 ..Last));
            end if;
        exit when Last < strItem'Last;
        end loop;
        bOK := bOK and (strLine /= Ada.Strings.Unbounded.Null_Unbounded_String);
    end read_line;

    procedure load_var(pValue: in System.Address; pfnStringToType: Smu_Types.PFN_STRING_TO_TYPE; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
        strLine : Ada.Strings.Unbounded.Unbounded_String;
        endptr : aliased System.Address := System.Null_Address;
        use type Interfaces.C.int;
        use type System.Address;
        use type Character;
    begin
        read_line(snsFile, strLine, bOK);
        if bOK then
            bOK := bOK and (pfnStringToType(Interfaces.C.Strings.new_string(Ada.Strings.Unbounded.To_String(strLine)), pValue, endptr'Access) /= 0)
                       and (endptr /=  System.Null_Address);
        end if ;
        declare
            endchar : Character; for endchar'Address use endptr;
        begin
            if bOK then
                bOK := bOK and (endchar = HT or endchar = ' ' or endchar = NUL);
            end if;
        end;
    end load_var;


    package body Smuw_Kcg_Top is

        package body Context_M3C3 is

            procedure save(adaPath: in String; pValue: in System.Address; pBaseAddr: in System.Address; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
                ctx: aliased Kcg_Top.Context_M3C3; for ctx'Address use pValue;
            begin
                save_field(adapath, ".MinOn3Cycles", ctx.MinOn3Cycles'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".MeanOn3Cycles", ctx.MeanOn3Cycles'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".MaxOn3Cycles", ctx.MaxOn3Cycles'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_context(adapath, ".SubCtx_MinMaxCycle3_1", ctx.SubCtx_MinMaxCycle3_1'Address, pBaseAddr, Smuw_Kcg_Top.Context_MinMaxCycle3.save'Access, snsFile, bOK);
                save_context(adapath, ".SubCtx_MeanCycle3_1", ctx.SubCtx_MeanCycle3_1'Address, pBaseAddr, Smuw_linear.Context_MeanCycle3_float64.save'Access, snsFile, bOK);
            end save;

            procedure load(value: out Kcg_Top.Context_M3C3; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
            begin
                load_var(value.MinOn3Cycles'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.MeanOn3Cycles'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.MaxOn3Cycles'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                Smuw_Kcg_Top.Context_MinMaxCycle3.load(value.SubCtx_MinMaxCycle3_1, snsFile, bOK);
                Smuw_linear.Context_MeanCycle3_float64.load(value.SubCtx_MeanCycle3_1, snsFile, bOK);
            end load;

        end Context_M3C3;
        package body Context_MinMaxCycle3 is

            procedure save(adaPath: in String; pValue: in System.Address; pBaseAddr: in System.Address; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
                ctx: aliased Kcg_Top.Context_MinMaxCycle3; for ctx'Address use pValue;
            begin
                save_field(adapath, ".MinOn3Cycles", ctx.MinOn3Cycles'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".MaxOn3Cycles", ctx.MaxOn3Cycles'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".mem_U", ctx.mem_U'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".L12", ctx.L12'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".init", ctx.init'Address, pBaseAddr, Smuw_Type.Smuw_Boolean.to_string'Access, snsFile, bOK);
            end save;

            procedure load(value: out Kcg_Top.Context_MinMaxCycle3; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
            begin
                load_var(value.MinOn3Cycles'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.MaxOn3Cycles'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.mem_U'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.L12'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.init'Address, Smuw_Type.Smuw_Boolean.string_to'Access, snsFile, bOK);
            end load;

        end Context_MinMaxCycle3;
    end Smuw_Kcg_Top;
    package body Smuw_linear is

        package body Context_MeanCycle3_float64 is

            procedure save(adaPath: in String; pValue: in System.Address; pBaseAddr: in System.Address; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
                ctx: aliased linear.Context_MeanCycle3_float64; for ctx'Address use pValue;
            begin
                save_field(adapath, ".MeanOn3Step_float64", ctx.MeanOn3Step_float64'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".mem_U_float64", ctx.mem_U_float64'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
                save_field(adapath, ".init", ctx.init'Address, pBaseAddr, Smuw_Type.Smuw_Boolean.to_string'Access, snsFile, bOK);
                save_field(adapath, ".L7_float64", ctx.L7_float64'Address, pBaseAddr, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.to_string'Access, snsFile, bOK);
            end save;

            procedure load(value: out linear.Context_MeanCycle3_float64; snsFile: in Ada.Text_IO.File_Type; bOK: in out Boolean) is
            begin
                load_var(value.MeanOn3Step_float64'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.mem_U_float64'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
                load_var(value.init'Address, Smuw_Type.Smuw_Boolean.string_to'Access, snsFile, bOK);
                load_var(value.L7_float64'Address, Smuw_Type.Smuw_Kcg_Config.Kcg_Float64.string_to'Access, snsFile, bOK);
            end load;

        end Context_MeanCycle3_float64;
    end Smuw_linear;

end Smuw_Snapshot;
