with Kcg_Config; use Kcg_Config;
with Smu_Predef_Conv; use Smu_Predef_Conv;
with Interfaces, Interfaces.C, Interfaces.C.Strings;
use type Interfaces.C.unsigned;
use type Interfaces.C.int;
with Ada.Strings; use Ada.Strings;
with Ada.Strings.Fixed; use Ada.Strings.Fixed;

package body Smuw_Type
is
    package body Smuw_Boolean is

        --****************************************************************
        --    Boolean
        --****************************************************************
        function to_string(pValue: System.Address; pfnStrAppend: Smu_Types.PFN_STR_APPEND; pStrObj: System.address) return Interfaces.C.int is
            value: Boolean; for value'Address use pValue;
        begin
            if Smu_vTable.type_to_string(pValue,vtable, pfnStrAppend, pStrObj) = 0 then
                return predef_Boolean_to_string(value, pfnStrAppend, pStrObj);
            end if;
            return 1;
        exception
            when others => return 0;
        end to_string;

        function check_string(str: Interfaces.C.Strings.chars_ptr; endptr: access System.Address) return Interfaces.C.int is
           temp : Boolean;
        begin
           return string_to(str, temp'address, endptr);
        end check_string;

        function string_to(szStr: Interfaces.C.Strings.chars_ptr; pValue: System.Address; endptr: access System.Address) return Interfaces.C.int is
            value: aliased Boolean; for value'Address use pValue;
            nRet: Interfaces.C.int := 1;
        begin
            if Smu_vTable.string_to_type(szStr, vtable, pValue, endptr) = 0 then
                return predef_string_to_Boolean(szStr, value'access, endptr);
            end if;
            return nRet;
        exception
            when others => return 0;
        end string_to;

        function is_double_conversion_allowed return Interfaces.C.int is
            nRet : Interfaces.C.int := 0;
        begin
            nRet := Smu_vTable.is_double_conversion_allowed(vtable);
            if nRet = -1 then
                nRet := 1;
            end if;
            return nRet;
        end is_double_conversion_allowed;

        function to_double(pValue: System.Address; retValue: access Interfaces.C.double) return Interfaces.C.int is
           value: Boolean; for value'address use pValue;
        begin
            if Smu_vTable.to_double(pValue, vtable, retValue) = 0 then
                if value then
                    retValue.all := 1.0;
                else
                    retValue.all := 0.0;
                end if; 
            end if;
            return 1;
        exception
            when others => return 0;
        end to_double;

        function is_long_conversion_allowed return Interfaces.C.int is
            nRet : Interfaces.C.int := 0;
        begin
            nRet := Smu_vTable.is_long_conversion_allowed(vtable);
            if nRet = -1 then
                nRet := 1;
            end if;
            return nRet;
        end is_long_conversion_allowed;

        function to_long(pValue: System.Address; retValue: access Interfaces.C.int) return Interfaces.C.int is
           value: Boolean; for value'address use pValue;
        begin
            if Smu_vTable.to_long(pValue, vtable, retValue) = 0 then
                if value then
                    retValue.all := 1;
                else
                    retValue.all := 0;
                end if; 
            end if;
                return 1;
            exception
                when others => return 0;
        end to_long;

        procedure compare(pResult: access Interfaces.C.unsigned; pValue1: System.address; pValue2: System.address; pTol: access Smu_Types.Sim_Tolerance; pszPath: Interfaces.C.Strings.chars_ptr; pfnStrListAppend: Smu_Types.PFN_STR_LIST_APPEND; pListErrPaths: System.address) is
            value1 : aliased Boolean; for value1'Address use pValue1;
            value2 : aliased Boolean; for value2'Address use pValue2;
            unitResult: aliased Interfaces.C.unsigned;
        begin
           if Smu_vTable.type_compare(unitResult'access, vtable, pValue1, pValue2, pTol) = 0 then
                unitResult := predef_compare_Boolean(value1, value2);
           end if;
           pResult.all := pResult.all or unitResult;
           Smu_vTable.type_compare_message(unitResult, pszPath, pfnStrListAppend, pListErrPaths);
        end compare;

        Sgn: constant Interfaces.C.Strings.chars_ptr := Interfaces.C.Strings.new_string("B");
        function get_signature(pfnStrAppend: Smu_Types.PFN_STR_APPEND; pStrObj: System.address) return Interfaces.C.int is
        begin
            return pfnStrAppend(Sgn, pStrObj);
        exception
            when others => return 0;
        end get_signature;

        function set_default_value(pValue: System.address) return Interfaces.C.int is
            value: Boolean := false; for value'address use pValue;
        begin
            return 1;
        exception
            when others => return 0;
        end set_default_value;

    end Smuw_Boolean;

    package body Smuw_Kcg_Config is

        package body Kcg_Char is

            --****************************************************************
            --    Kcg_Config.Kcg_Char
            --****************************************************************
            function to_string(pValue: System.Address; pfnStrAppend: Smu_Types.PFN_STR_APPEND; pStrObj: System.address) return Interfaces.C.int is
                value: Kcg_Config.Kcg_Char; for value'Address use pValue;
            begin
                if Smu_vTable.type_to_string(pValue,vtable, pfnStrAppend, pStrObj) = 0 then
                    return predef_Kcg_Char_to_string(value, pfnStrAppend, pStrObj);
                end if;
                return 1;
            exception
                when others => return 0;
            end to_string;

            function check_string(str: Interfaces.C.Strings.chars_ptr; endptr: access System.Address) return Interfaces.C.int is
               temp : Kcg_Config.Kcg_Char;
            begin
               return string_to(str, temp'address, endptr);
            end check_string;

            function string_to(szStr: Interfaces.C.Strings.chars_ptr; pValue: System.Address; endptr: access System.Address) return Interfaces.C.int is
                value: aliased Kcg_Config.Kcg_Char; for value'Address use pValue;
                nRet: Interfaces.C.int := 1;
            begin
                if Smu_vTable.string_to_type(szStr, vtable, pValue, endptr) = 0 then
                    return predef_string_to_Kcg_Char(szStr, value'access, endptr);
                end if;
                return nRet;
            exception
                when others => return 0;
            end string_to;

            function is_double_conversion_allowed return Interfaces.C.int is
                nRet : Interfaces.C.int := 0;
            begin
                nRet := Smu_vTable.is_double_conversion_allowed(vtable);
                if nRet = -1 then
                    nRet := 1;
                end if;
                return nRet;
            end is_double_conversion_allowed;

            function to_double(pValue: System.Address; retValue: access Interfaces.C.double) return Interfaces.C.int is
               value: Kcg_Config.Kcg_Char; for value'address use pValue;
            begin
                if Smu_vTable.to_double(pValue, vtable, retValue) = 0 then
                    retValue.all := Interfaces.C.double(Kcg_Config.Kcg_Char'Pos(value));
                end if;
                return 1;
            exception
                when others => return 0;
            end to_double;

            function is_long_conversion_allowed return Interfaces.C.int is
                nRet : Interfaces.C.int := 0;
            begin
                nRet := Smu_vTable.is_long_conversion_allowed(vtable);
                if nRet = -1 then
                    nRet := 1;
                end if;
                return nRet;
            end is_long_conversion_allowed;

            function to_long(pValue: System.Address; retValue: access Interfaces.C.int) return Interfaces.C.int is
               value: Kcg_Config.Kcg_Char; for value'address use pValue;
            begin
                if Smu_vTable.to_long(pValue, vtable, retValue) = 0 then
                    retValue.all := Interfaces.C.int(Kcg_Config.Kcg_Char'Pos(value));
                end if;
                    return 1;
                exception
                    when others => return 0;
            end to_long;

            procedure compare(pResult: access Interfaces.C.unsigned; pValue1: System.address; pValue2: System.address; pTol: access Smu_Types.Sim_Tolerance; pszPath: Interfaces.C.Strings.chars_ptr; pfnStrListAppend: Smu_Types.PFN_STR_LIST_APPEND; pListErrPaths: System.address) is
                value1 : aliased Kcg_Config.Kcg_Char; for value1'Address use pValue1;
                value2 : aliased Kcg_Config.Kcg_Char; for value2'Address use pValue2;
                unitResult: aliased Interfaces.C.unsigned;
            begin
               if Smu_vTable.type_compare(unitResult'access, vtable, pValue1, pValue2, pTol) = 0 then
                    unitResult := predef_compare_Kcg_Char(value1, value2);
               end if;
               pResult.all := pResult.all or unitResult;
               Smu_vTable.type_compare_message(unitResult, pszPath, pfnStrListAppend, pListErrPaths);
            end compare;

            Sgn: constant Interfaces.C.Strings.chars_ptr := Interfaces.C.Strings.new_string("C");
            function get_signature(pfnStrAppend: Smu_Types.PFN_STR_APPEND; pStrObj: System.address) return Interfaces.C.int is
            begin
                return pfnStrAppend(Sgn, pStrObj);
            exception
                when others => return 0;
            end get_signature;

            function set_default_value(pValue: System.address) return Interfaces.C.int is
                value: Kcg_Config.Kcg_Char := Kcg_Config.Kcg_Char'First; for value'address use pValue;
            begin
                return 1;
            exception
                when others => return 0;
            end set_default_value;

        end Kcg_Char;
        package body Kcg_Float32 is

        end Kcg_Float32;
        package body Kcg_Float64 is

        end Kcg_Float64;
        package body Kcg_Int16 is

        end Kcg_Int16;
        package body Kcg_Int32 is

        end Kcg_Int32;
        package body Kcg_Int64 is

        end Kcg_Int64;
        package body Kcg_Int8 is

        end Kcg_Int8;
        package body Kcg_Size is

            --****************************************************************
            --    Kcg_Config.Kcg_Size
            --****************************************************************
            function check_string(str: Interfaces.C.Strings.chars_ptr; endptr: access System.Address) return Interfaces.C.int is
               temp : Kcg_Config.Kcg_Size;
            begin
               return string_to(str, temp'address, endptr);
            end check_string;

            function string_to(szStr: Interfaces.C.Strings.chars_ptr; pValue: System.Address; endptr: access System.Address) return Interfaces.C.int is
                value: aliased Kcg_Config.Kcg_Size; for value'Address use pValue;
                nRet: Interfaces.C.int := 1;
            begin
                if Smu_vTable.string_to_type(szStr, vtable, pValue, endptr) = 0 then
                    return predef_string_to_kcg_size(szStr, value'access, endptr);
                end if;
                return nRet;
            exception
                when others => return 0;
            end string_to;

            function is_double_conversion_allowed return Interfaces.C.int is
                nRet : Interfaces.C.int := 0;
            begin
                nRet := Smu_vTable.is_double_conversion_allowed(vtable);
                if nRet = -1 then
                    nRet := 1;
                end if;
                return nRet;
            end is_double_conversion_allowed;

            function to_double(pValue: System.Address; retValue: access Interfaces.C.double) return Interfaces.C.int is
               value: Kcg_Config.Kcg_Size; for value'address use pValue;
            begin
                if Smu_vTable.to_double(pValue, vtable, retValue) = 0 then
                    retValue.all := Interfaces.C.double(value);
                end if;
                return 1;
            exception
                when others => return 0;
            end to_double;

            function is_long_conversion_allowed return Interfaces.C.int is
                nRet : Interfaces.C.int := 0;
            begin
                nRet := Smu_vTable.is_long_conversion_allowed(vtable);
                if nRet = -1 then
                    nRet := 1;
                end if;
                return nRet;
            end is_long_conversion_allowed;

            function to_long(pValue: System.Address; retValue: access Interfaces.C.int) return Interfaces.C.int is
               value: Kcg_Config.Kcg_Size; for value'address use pValue;
            begin
                if Smu_vTable.to_long(pValue, vtable, retValue) = 0 then
                    retValue.all := Interfaces.C.int(value);
                end if;
                    return 1;
                exception
                    when others => return 0;
            end to_long;

            procedure compare(pResult: access Interfaces.C.unsigned; pValue1: System.address; pValue2: System.address; pTol: access Smu_Types.Sim_Tolerance; pszPath: Interfaces.C.Strings.chars_ptr; pfnStrListAppend: Smu_Types.PFN_STR_LIST_APPEND; pListErrPaths: System.address) is
                value1 : aliased Kcg_Config.Kcg_Size; for value1'Address use pValue1;
                value2 : aliased Kcg_Config.Kcg_Size; for value2'Address use pValue2;
                unitResult: aliased Interfaces.C.unsigned;
            begin
               if Smu_vTable.type_compare(unitResult'access, vtable, pValue1, pValue2, pTol) = 0 then
                    unitResult := predef_compare_kcg_size(value1, value2);
               end if;
               pResult.all := pResult.all or unitResult;
               Smu_vTable.type_compare_message(unitResult, pszPath, pfnStrListAppend, pListErrPaths);
            end compare;

            Sgn: constant Interfaces.C.Strings.chars_ptr := Interfaces.C.Strings.new_string("S");
            function get_signature(pfnStrAppend: Smu_Types.PFN_STR_APPEND; pStrObj: System.address) return Interfaces.C.int is
            begin
                return pfnStrAppend(Sgn, pStrObj);
            exception
                when others => return 0;
            end get_signature;

            function set_default_value(pValue: System.address) return Interfaces.C.int is
                value: Kcg_Config.Kcg_Size := 0; for value'address use pValue;
            begin
                return 1;
            exception
                when others => return 0;
            end set_default_value;

        end Kcg_Size;
        package body Kcg_Uint16 is

        end Kcg_Uint16;
        package body Kcg_Uint32 is

        end Kcg_Uint32;
        package body Kcg_Uint64 is

        end Kcg_Uint64;
        package body Kcg_Uint8 is

        end Kcg_Uint8;
    end Smuw_Kcg_Config;
    package body Smuw_Kcg_Top is

        package body Context_M3C3 is

            fields_intialized: Boolean := False;
            function fields return Smu_Types.Array_Sim_Field_Utils is 
            begin
                if fields_intialized = False then
                    fields_data := (
                        (Interfaces.C.Strings.new_string("MinOn3Cycles"), object.MinOn3Cycles'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("MeanOn3Cycles"), object.MeanOn3Cycles'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("MaxOn3Cycles"), object.MaxOn3Cycles'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("SubCtx_MinMaxCycle3_1"), object.SubCtx_MinMaxCycle3_1'position, null), 
                        (Interfaces.C.Strings.new_string("SubCtx_MeanCycle3_1"), object.SubCtx_MeanCycle3_1'position, null)
                    );
                    fields_intialized := True;
                end if;
                return fields_data;
            end fields;
        end Context_M3C3;
        package body Context_MinMaxCycle3 is

            fields_intialized: Boolean := False;
            function fields return Smu_Types.Array_Sim_Field_Utils is 
            begin
                if fields_intialized = False then
                    fields_data := (
                        (Interfaces.C.Strings.new_string("MinOn3Cycles"), object.MinOn3Cycles'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("MaxOn3Cycles"), object.MaxOn3Cycles'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("mem_U"), object.mem_U'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("L12"), object.L12'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("init"), object.init'position, Smuw_Boolean.Type_Utils'access)
                    );
                    fields_intialized := True;
                end if;
                return fields_data;
            end fields;
        end Context_MinMaxCycle3;
    end Smuw_Kcg_Top;
    package body Smuw_linear is

        package body Context_MeanCycle3_float64 is

            fields_intialized: Boolean := False;
            function fields return Smu_Types.Array_Sim_Field_Utils is 
            begin
                if fields_intialized = False then
                    fields_data := (
                        (Interfaces.C.Strings.new_string("MeanOn3Step_float64"), object.MeanOn3Step_float64'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("mem_U_float64"), object.mem_U_float64'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("init"), object.init'position, Smuw_Boolean.Type_Utils'access), 
                        (Interfaces.C.Strings.new_string("L7_float64"), object.L7_float64'position, Smuw_Kcg_Config.Kcg_Float64.Type_Utils'access)
                    );
                    fields_intialized := True;
                end if;
                return fields_data;
            end fields;
        end Context_MeanCycle3_float64;
    end Smuw_linear;

end Smuw_Type;
